package com.winhc.task.job;

import com.mongodb.client.MongoCollection;
//import com.winhc.phoenix.example.framework.hbase.HbaseScan;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.Table;
import org.bson.Document;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/2/1 16:54
 */
@Slf4j
@Component
@AllArgsConstructor
public class CompanyDynamicFix {
    private final Connection connection;
    private final MongoTemplate mongoTemplate;

    @Getter
    @Builder
    @ToString
    private static class DynamicInfo {
        private String id;
        private String tn;
        private String rowkey;

    }

    public void start() {
        MongoCollection<Document> xjk_fix_company_dynamic = mongoTemplate.getCollection("xjk_fix_company_dynamic");
        Consumer<List<Map<String, String>>> func = list -> {
            List<DynamicInfo> dynamicInfos = list.stream()
                    .map(d -> DynamicInfo.builder().id(d.get("rowkey")).tn(d.get("F:INFO_TYPE")).rowkey(d.get("F:BIZ_ID")).build())
                    .collect(Collectors.toList());

            List<Document> dos = new ArrayList<>();

            Map<String, List<DynamicInfo>> collect = dynamicInfos.stream().collect(Collectors.groupingBy(DynamicInfo::getTn));
            for (Map.Entry<String, List<DynamicInfo>> entry : collect.entrySet()) {
                if (entry.getKey().equalsIgnoreCase("company_employment")) {
                    continue;
                }
                if (entry.getKey().equalsIgnoreCase("wenshu_detail_combine")) {
                    continue;
                }
                if (entry.getKey().equalsIgnoreCase("intellectual")) {
                    continue;
                }
                List<DynamicInfo> value = entry.getValue();
                TableName tn = TableName.valueOf(entry.getKey().toUpperCase());
                try (Table table = connection.getTable(tn)) {
                    boolean[] exists = table.exists(value.stream().map(DynamicInfo::getRowkey).map(String::getBytes).map(Get::new).collect(Collectors.toList()));
                    for (int i = 0; i < exists.length; i++) {
                        if (!exists[i]) {
                            DynamicInfo dynamicInfo = value.get(i);
                            Document document = new Document();
                            document.put("_id", dynamicInfo.getId());
                            document.put("tn", dynamicInfo.getTn());
                            document.put("rowkey", dynamicInfo.getRowkey());
                            dos.add(document);
                        }
                    }

                } catch (Exception e) {
                    log.error("tn不存在：{}", entry.getKey());
                }
            }
            log.info("\t{}", dos);
            if (!dos.isEmpty()) {
                xjk_fix_company_dynamic.insertMany(dos);
            }
        };

        //new HbaseScan(connection, "COMPANY_DYNAMIC", func).columns("F:INFO_TYPE,F:BIZ_ID").batchSize(1000).scan();
    }
}
