package com.winhc.task.job;

import com.winhc.task.framework.es.EsFastScan;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.InnerHitBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.collapse.CollapseBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.stereotype.Component;
import java.io.IOException;
import java.util.List;
import java.util.function.Consumer;


@Slf4j
@Component
@AllArgsConstructor
public class EsQueryListJobs {
    private RestHighLevelClient restHighLevelClient;

    public void start(String index, String type) throws IOException {
//        Consumer<SearchHit[]> func = list -> {
//            ArrayList<String> l = new ArrayList<>();
//            Arrays.stream(list).forEach(d -> {
//                l.add(d.getId());
//            });
//            try {
//                query(index, type, l);
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//        };
//        queryIds(func);
    }

    public void queryIds(Consumer<SearchHit[]> func) {
        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"match_all\": {}\n" +
                "  }\n" +
                "}";
        new EsFastScan(restHighLevelClient, func, "winhc-company-v8_4", "company", dsl, 1000, 1).scan();
    }

    public void query(String index, String type, List<String> companyIds) throws IOException {
        long start = System.currentTimeMillis();
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery()
                .should(QueryBuilders.termsQuery("company_id", companyIds));
        FetchSourceContext fetchSourceContext = new FetchSourceContext(true, null, null);
        CollapseBuilder collapseBuilder = new CollapseBuilder("company_id");
        InnerHitBuilder innerHitBuilder = new InnerHitBuilder().setName("company_agg")
                .setFrom(0)
                .setSize(100)
                .setTrackScores(true)
                .setIgnoreUnmapped(true)
                .setFetchSourceContext(fetchSourceContext)
                .addSort(SortBuilders.fieldSort("company_id")
                        .order(SortOrder.DESC));
        collapseBuilder.setInnerHits(innerHitBuilder);
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(queryBuilder)
                .from(0)
                .size(companyIds.size())
                .sort("company_id", SortOrder.DESC)
                .collapse(collapseBuilder);

        SearchRequest searchRequest = new SearchRequest()
                .indices(index).types(type)
                .source(searchSourceBuilder);
        SearchResponse response = restHighLevelClient.search(searchRequest);
        System.out.println(response.toString());
        log.info("query size: {} , cost: {}", companyIds.size(), System.currentTimeMillis() - start);
    }


}
