package com.winhc.task.job;

import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.InsertManyOptions;
import com.mongodb.client.result.InsertManyResult;
import com.winhc.task.framework.es.EsFastScan;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.bson.BsonValue;
import org.bson.Document;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.search.SearchHit;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.stream.Collectors;


@Slf4j
@Component
@AllArgsConstructor
public class EsScanCompanyHumanRelationMapping {
    private RestHighLevelClient restHighLevelClient;
    private final MongoTemplate mongoTemplate;

    public void start() {
        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"bool\": {\n" +
                "      \"must\": [\n" +
                "        {\n" +
                "          \"exists\": {\n" +
                "            \"field\": \"yhc_type\"\n" +
                "          }\n" +
                "        }\n" +
                "      ]\n" +
                "    }\n" +
                "  }\n" +
                "}";
        scan(dsl);
    }

    public void scan(String dsl) {
        MongoCollection<Document> person = mongoTemplate.getCollection("xf_company_human_pid_mapping_incr");
        Consumer<SearchHit[]> func = list -> {
            List<Document> li = Arrays.stream(list).map(d -> {
                Map<String, Object> m1 = d.getSourceAsMap();
                String id = d.getId();
                Document document = new Document();
                document.put("_id", id);
                document.put("company_id", m1.getOrDefault("company_id", ""));
                document.put("human_name", m1.getOrDefault("human_name_cleanup", ""));
                document.put("human_pid", m1.getOrDefault("human_pid", ""));
                //document.put("_id", id);
                return document;
            }).filter(x -> x.get("_id").toString().length() == 32)
                    .distinct().collect(Collectors.toList());
            InsertManyResult insertManyResult = person.insertMany(li, new InsertManyOptions().ordered(false));
            Map<Integer, BsonValue> m = insertManyResult.getInsertedIds();
            System.out.println(m.size());
        };
        new EsFastScan(restHighLevelClient, func, "company-human-pid-mapping-v3", "human", dsl, null).scan();
    }

}
