package com.winhc.task.service.impl;

import com.winhc.task.service.HbaseOperationService;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.Delete;
import org.apache.hadoop.hbase.client.Table;
import org.springframework.stereotype.Service;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2020/10/21 09:55
 */
@Slf4j
@Service
@AllArgsConstructor
public class HbaseOperationServiceImpl implements HbaseOperationService {

    private final Connection connection;

    @SneakyThrows
    @Override
    public boolean deleteByRowkey(String tableName, List<String> ids) {
        List<Delete> deletes = ids.stream()
                .map(String::getBytes)
                .map(Delete::new)
                .collect(Collectors.toList());
        try (Table table = connection.getTable(TableName.valueOf(tableName))) {
            log.info("deletes size:{}", deletes.size());
            table.delete(deletes);
        }
        return true;
    }

    @SneakyThrows
    @Override
    public boolean deleteByRowkey(String tableName, String id) {
        try (Table table = connection.getTable(TableName.valueOf(tableName))) {
            Delete delete = new Delete(id.getBytes());
            table.delete(delete);
        }
        return true;
    }

    @SneakyThrows
    @Override
    public boolean deleteColsByRowkey(String tableName, List<String> ids, String family, String qualifier) {
        final byte[] f = family.getBytes();
        final byte[] c = qualifier.getBytes();
        List<Delete> deletes = ids.stream()
                .map(String::getBytes)
                .map(Delete::new)
                .peek(delete -> delete.addColumns(f, c))
                .collect(Collectors.toList());
        try (Table table = connection.getTable(TableName.valueOf(tableName))) {
            log.info("deletes size:{}", deletes.size());
            table.delete(deletes);
        }
        return true;
    }

    @SneakyThrows
    @Override
    public boolean deleteColsByRowkey(String tableName, String id, String family, String qualifier) {
        try (Table table = connection.getTable(TableName.valueOf(tableName))) {
            Delete delete = new Delete(id.getBytes());
            delete.addColumns(family.getBytes(), qualifier.getBytes());
            table.delete(delete);
        }
        return true;
    }
}
