package com.winhc.task.service.impl;

import com.aliyun.openservices.ons.api.Producer;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagRequest;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.aliyuncs.dataworks_public.model.v20180601.SearchManualDagNodeInstanceRequest;
import com.aliyuncs.dataworks_public.model.v20180601.SearchManualDagNodeInstanceResponse;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.http.ProtocolType;

import com.winhc.task.bean.TaskFlowEnum;
import com.winhc.task.bean.TaskParam;
import com.winhc.task.configuration.DataWorksAccessProperties;
import com.winhc.task.service.TouchService;
import com.winhc.task.util.JsonUtils;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/22 11:28
 * @Description:
 */
@Slf4j
@Service
@AllArgsConstructor
public class TouchServiceImpl implements TouchService {

    private final DataWorksAccessProperties dataWorksAccessProperties;
    private final IAcsClient client;

    @SneakyThrows
    public CreateManualDagResponse touch(String projectName
            , String flowName, String bizDate){
        log.info("触发任务：{}.{} {}", projectName, flowName, bizDate);
        CreateManualDagRequest request = new CreateManualDagRequest();
        request.setProjectName(projectName);
        request.setFlowName(flowName);
        request.setBizdate(bizDate + " 00:00:00");
        request.setRegionId(dataWorksAccessProperties.getRegionId());
        request.setProtocol(ProtocolType.HTTP);
        CreateManualDagResponse response = client.getAcsResponse(request);
        log.info("\n任务结果：\n\trequestId:{},\n\treturnCode:{},\n\treturnErrorSolution:{},\n\treturnMessage:{},\n\treturnValue:{}",
                response.getRequestId()
                , response.getReturnCode()
                , response.getReturnErrorSolution()
                , response.getReturnMessage()
                , response.getReturnValue());
        return response;
    }

    @SneakyThrows
    public CreateManualDagResponse triggerWithParam(String projectName
            , String flowName, String bizDate, String dagPara, String nodePara) {
        log.info("触发任务：{}.{} {}", projectName, flowName, bizDate);
        CreateManualDagRequest request = new CreateManualDagRequest();
        request.setProjectName(projectName);
        request.setFlowName(flowName);
        request.setBizdate(bizDate + " 00:00:00");
        request.setRegionId(dataWorksAccessProperties.getRegionId());
        request.setProtocol(ProtocolType.HTTP);
        request.setDagPara(dagPara);
        request.setNodePara(nodePara);
        CreateManualDagResponse response = client.getAcsResponse(request);
        log.info("\n任务结果：\n\trequestId:{},\n\treturnCode:{},\n\treturnErrorSolution:{},\n\treturnMessage:{},\n\treturnValue:{}",
                response.getRequestId()
                , response.getReturnCode()
                , response.getReturnErrorSolution()
                , response.getReturnMessage()
                , response.getReturnValue());
        return response;
    }

    @SneakyThrows
    public CreateManualDagResponse touch(TaskParam taskParam) {
        log.info("触发任务：{}", taskParam.toString());
        CreateManualDagRequest request = new CreateManualDagRequest();
        request.setProjectName(taskParam.getProjectName());
        request.setFlowName(taskParam.getFlowName());
        request.setBizdate(taskParam.getBizDate() + " 00:00:00");
        request.setRegionId(dataWorksAccessProperties.getRegionId());
        request.setProtocol(ProtocolType.HTTP);
        request.setNodePara(JsonUtils.jsonObjToString(taskParam.getNodeParam()));
        CreateManualDagResponse response = client.getAcsResponse(request);
        log.info("\n任务结果：\n\trequestId:{},\n\treturnCode:{},\n\treturnErrorSolution:{},\n\treturnMessage:{},\n\treturnValue:{}",
                response.getRequestId()
                , response.getReturnCode()
                , response.getReturnErrorSolution()
                , response.getReturnMessage()
                , response.getReturnValue());
        return response;
    }


    @SneakyThrows
    public Map<String, TaskFlowEnum> query(String projectName, Long dagId) {
        SearchManualDagNodeInstanceRequest searchNodeInstanceListRequest
                = new SearchManualDagNodeInstanceRequest();
        searchNodeInstanceListRequest.setDagId(dagId);
        searchNodeInstanceListRequest.setProjectName(projectName);
        searchNodeInstanceListRequest.setProtocol(ProtocolType.HTTP);
        SearchManualDagNodeInstanceResponse searchResponse = client
                .getAcsResponse(searchNodeInstanceListRequest);
        java.util.List<SearchManualDagNodeInstanceResponse.NodeInsInfo> nodeInsfos = searchResponse.getData();
        for (SearchManualDagNodeInstanceResponse.NodeInsInfo nodeInsInfo : nodeInsfos) {
            TaskFlowEnum code = TaskFlowEnum.getTaskFlowEnumByCode(nodeInsInfo.getStatus());
            log.info("{}:{} {}", nodeInsInfo.getNodeName(), nodeInsInfo.getStatus(), code);
        }
        return nodeInsfos.stream()
                .collect(Collectors.toMap(SearchManualDagNodeInstanceResponse.NodeInsInfo::getNodeName, node -> TaskFlowEnum.getTaskFlowEnumByCode(node.getStatus()), (o1, o2) -> o1));
    }

}
