package com.winhc.task.service.impl;

import com.winhc.task.configuration.KafkaConfiguration;
import com.winhc.task.service.WinhcKafkaService;
import com.winhc.task.util.UnicodeBackslashU;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Service;
import org.springframework.util.concurrent.ListenableFuture;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author: XuJiakai
 * 2020/12/21 17:44
 */
@Slf4j
@Service
@AllArgsConstructor
@ConditionalOnProperty(value = "spring.profiles.active", havingValue = "prod", matchIfMissing = true)
public class WinhcKafkaServiceImpl implements WinhcKafkaService {
    private final KafkaConfiguration kafkaConfiguration;
    private final KafkaTemplate<String, String> kafkaTemplate;

    @Override
    public boolean send(String topic, String body) {
        ListenableFuture<SendResult<String, String>> send = kafkaTemplate.send(topic, body);
        return true;
    }

    @Override
    public Object getMsg(String topic, Integer maxSize) {
        Map<String, Object> stringObjectMap = kafkaConfiguration.consumerConfigs();
        stringObjectMap.put(ConsumerConfig.MAX_POLL_RECORDS_CONFIG, maxSize);
        KafkaConsumer<String, String> kafkaConsumer = new KafkaConsumer<>(stringObjectMap);
        List<String> subscribedTopics = new ArrayList<String>();
        subscribedTopics.add(topic);
        kafkaConsumer.subscribe(subscribedTopics);
        ConsumerRecords<String, String> poll = kafkaConsumer.poll(Duration.ofSeconds(5));
        List<String> rs = new ArrayList<>();

        for (ConsumerRecord<String, String> record : poll) {
            String value = record.value();
            rs.add(UnicodeBackslashU.unicodeToCn(value));
        }
        kafkaConsumer.commitAsync();
        kafkaConsumer.unsubscribe();
        return rs;
    }
}
