package com.winhc.task.util;

import org.apache.commons.codec.binary.Hex;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.AlgorithmParameters;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;


/**
 * AES加密模式：CBC
 * 填充【java中pkcs5等效果于pkcs7】：pkcs7
 * 数据块：128位（16字节）
 * 密码(key)【128/192/256比特，16/24/32字符】：winhc_168_000000
 * 偏移量(iv)【ECB模式只用密钥即可对数据进行加密解密，CBC模式需要添加一个参数iv】：afasdfasdfasdfyu
 * 输出：hex
 * 字符集：utf-8
 */

/**
 * @author: XuJiakai
 * 2021/2/3 11:44
 */
public class WinhcCryptoNativeCBC {

    private static final String key = "winhc_168_000000";
    private static final String iv = "afasdfasdfasdfyu";


    private static final String CIPHER_ALGORITHM = "AES/CBC/PKCS5Padding";
    private final Lock lock = new ReentrantLock();

    private final Cipher cipher;
    private final SecretKey secretKey;
    private AlgorithmParameters params;


    public WinhcCryptoNativeCBC() {
        try {
            cipher = Cipher.getInstance(CIPHER_ALGORITHM);
            secretKey = new SecretKeySpec(key.getBytes(StandardCharsets.UTF_8), "AES");
            params = AlgorithmParameters.getInstance("AES");
            params.init(new IvParameterSpec(iv.getBytes(StandardCharsets.UTF_8)));
            cipher.init(Cipher.ENCRYPT_MODE, secretKey, params);

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private byte[] encrypt(byte[] data) {
        lock.lock();
        try {
//            cipher.init(Cipher.ENCRYPT_MODE, secretKey, params);
            return cipher.doFinal(data);
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            lock.unlock();
        }
    }

    private byte[] decrypt(byte[] bytes) {
        lock.lock();
        try {
            cipher.init(Cipher.DECRYPT_MODE, secretKey, params);
            return cipher.doFinal(bytes);
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            lock.unlock();
        }
    }

    public String encryptHex(String data) {
        return Hex.encodeHexString(encrypt(data.getBytes(StandardCharsets.UTF_8)));
    }

    public String decryptStr(String data) {
        try {
            byte[] bytes = Hex.decodeHex(data);
            return new String(decrypt(bytes), StandardCharsets.UTF_8);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }


    public static void main(String[] args) {
        WinhcCryptoNativeCBC winhcCryptoNative = new WinhcCryptoNativeCBC();
        List<String> ids = Arrays.asList("12010119810111007X", "120225198108240076");
        for (int i = 0; i < 1000; i++) {
            String res = winhcCryptoNative.encryptHex("12010119810111007X");
            if ("bcb789e6022c4053a02d046327a639c10da537023d8dbe575efa7b67c2d4b5ee".equalsIgnoreCase(res) || "ab71045fe95da3d2f450a4bbf50428572bd8b59a58cd531eaa79a449adbd8af1".equalsIgnoreCase(res)) {
                System.out.println("相同");
            }else {
                System.out.println("不同");
            }
        }

//        System.out.println(winhcCryptoNative.decryptStr("55C0002A2E2CFD1EEC98FDD4B80E5B4D"));
    }
}
